/* ********************************************************* */
/*     readfits.c                                            */
/*     CIN Source File to Read data from FITS image file     */
/*     2003/02/04   YOSHIKAWA, Tomohiro                      */
/* ********************************************************* */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "extcode.h"
#include "fitsio.h"

#define DATATYPE TULONG      /* datatype */
#define ParamNumber 1        /* ImageData is parameter 1 */

/*
 * typedefs
 */

typedef struct {
	int32 dimSizes[2];
	uInt32 Numeric[1];
	} TD1;
typedef TD1 **TD1Hdl;

typedef struct {
	LVBoolean status;
	int32 code;
	LStrHandle source;
	} TD2;

CIN MgErr CINRun(LStrHandle readFile, TD1Hdl ImageData, TD2 *error);

CIN MgErr CINRun(LStrHandle readFile, TD1Hdl ImageData, TD2 *error) {
  fitsfile *fptr;              /* pointer to fitsfile structure */
  int status = 0;              /* returned error status code */
  long naxes[2];
  int  nfound;                 /* variable for fits_read_keys_lng() */
  long firstelem, nbuffer, nullval; 
  int anynull;                 /* variable for fits_read_img() */

  Str255 TmpPStrp;
  uChar readFileC[256];
  uChar err_text[30];

  MgErr err = noErr;

/* copy outFile to C type String */
  if(LStrLen(*readFile)>=256) {
    error->status = LVTRUE;
    LStrPrintf(error->source, "file path is too long.\n");
    goto end;
  }
  LToPStr(*readFile, TmpPStrp);
  PToCStr(TmpPStrp, readFileC);

/* open fits file */
  if(fits_open_file(&fptr, readFileC, READONLY, &status)) {
    fits_get_errstatus(status, err_text);
    error->status = LVTRUE;
    error->code = status;
    LStrPrintf(error->source, err_text);
    goto end;
  }

/* read the NAXIS1 and NAXIS2 keyword to get image size */
  if(fits_read_keys_lng(fptr, "NAXIS", 1, 2, naxes, &nfound, &status)) {
    fits_get_errstatus(status, err_text);
    error->status = LVTRUE;
    error->code = status;
    LStrPrintf(error->source, err_text);
    goto end;
  }
  nbuffer = naxes[0] * naxes[1];
  firstelem = 1;
  nullval = 0;

/* resize image data buffer */
  if(err = SetCINArraySize((UHandle)ImageData, ParamNumber, (int32)nbuffer)) {
    error->status = LVTRUE;
    LStrPrintf(error->source, "Set array size error.\n");
    goto end;
  }
  (*ImageData)->dimSizes[1] = naxes[0];
  (*ImageData)->dimSizes[0] = naxes[1];

/* read fits file */
  if(fits_read_img(fptr, DATATYPE, firstelem, nbuffer, &nullval, (*ImageData)->Numeric, &anynull, &status)) {
    fits_get_errstatus(status, err_text);
    error->status = LVTRUE;
    error->code = status;
    LStrPrintf(error->source, err_text);
    goto end;
  }

/* close fits file */
  if(fits_close_file(fptr, &status)) {
    fits_get_errstatus(status, err_text);
    error->status = LVTRUE;
    error->code = status;
    LStrPrintf(error->source, err_text);
    goto end;
  }

  end: ;
  return noErr;
}












