/* ********************************************************* */
/*     writefits.c                                           */
/*     CIN Source File to Write FITS image file              */
/*     2003/1/10   YOSHIKAWA, Tomohiro                       */
/* ********************************************************* */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "extcode.h"
#include "fitsio.h"

#define BITPIX ULONG_IMG     /* bit per pixel */
#define DATATYPE TULONG      /* datatype */

/*
 * typedefs
 */

typedef struct {
	int32 dimSizes[2];
	uInt32 Numeric[1];
	} TD1;
typedef TD1 **TD1Hdl;

typedef struct {
	LVBoolean status;
	int32 code;
	LStrHandle source;
	} TD2;

CIN MgErr CINRun(LStrHandle outFile, TD1Hdl ImageData, TD2 *error);

CIN MgErr CINRun(LStrHandle outFile, TD1Hdl ImageData, TD2 *error) {
  fitsfile *fptr;              /* pointer to fitsfile structure */
  int status = 0;              /* returned error status code */
  long naxis, naxes[2];        /* variable for fits_create_img() */
  long firstelem, nelements;   /* variable for fits_write_img() */

  Str255 TmpPStrp;
  uChar outFileC[256];
  uChar err_text[30];

/* copy outFile to C type String */
  if(LStrLen(*outFile)>=256) {
    error->status = LVTRUE;
    LStrPrintf(error->source, "file path is too long.\n");
    goto end;
  }
  LToPStr(*outFile, TmpPStrp);
  PToCStr(TmpPStrp, outFileC);

/* create and open a fits file */	
  if(fits_create_file(&fptr, outFileC, &status)) {
    fits_get_errstatus(status, err_text);
    error->status = LVTRUE;
    error->code = status;
    LStrPrintf(error->source, err_text);
    goto end;
  }

  naxis = 2;                            /* number of dimentions */
  naxes[0] = (*ImageData)->dimSizes[1];   /* length of the X axis */
  naxes[1] = (*ImageData)->dimSizes[0];   /* length of the Y axis */

/* create a fits image */
  if(fits_create_img(fptr, BITPIX, naxis, naxes, &status)) {
    fits_get_errstatus(status, err_text);
    error->status = LVTRUE;
    error->code = status;
    LStrPrintf(error->source, err_text);
    goto end;
 }

  firstelem = 1;                        /* first element in a vector(????) */
  nelements = naxes[0] * naxes[1];      /* number of data elements */

/* write elements into the fits data array */
  if(fits_write_img(fptr, DATATYPE, firstelem, nelements, (*ImageData)->Numeric, &status)) {
    fits_get_errstatus(status, err_text);
    error->status = LVTRUE;
    error->code = status;
    LStrPrintf(error->source, err_text);
    goto end;
  }

/* close the fits file */
  if(fits_close_file(fptr, &status)) {
    fits_get_errstatus(status, err_text);
    error->status = LVTRUE;
    error->code = status;
    LStrPrintf(error->source, err_text);
    goto end;
  }
  end: ;
  return noErr;
}












